<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://your-style.nl
 * @since      1.0.0
 *
 * @package    Yourstyle_Plus_Companion_Plugin
 * @subpackage Yourstyle_Plus_Companion_Plugin/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Yourstyle_Plus_Companion_Plugin
 * @subpackage Yourstyle_Plus_Companion_Plugin/includes
 * @author     Bjorn Manintveld <bjorn@yourstyledesign.nl>
 */

class Yourstyle_Plus_Companion_Plugin
{

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
    protected $plugin_name;
    
	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
    protected $version;
    
	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
    public function __construct()
    {
		if (defined('YOURSTYLE_PLUS_COMPANION_PLUGIN')) {
			$this->version = YOURSTYLE_PLUS_COMPANION_PLUGIN;
		} else {
			$this->version = '1.0.0';
		}
        $this->plugin_name = 'yourstyle-plus-companion-plugin';
        $this->load_addons();
        $this->load_admin_class();
        add_action( 'admin_enqueue_scripts', array($this, 'load_admin_style') );
    }

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
    public function get_plugin_name()
    {
		return $this->plugin_name;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
    public function get_version()
    {
		return $this->version;
    }

	/**
	 * Load the Admin functionality of the plugin.
	 * Load the widgets for the yourstyle-plus dashboard.
	 *
	 * @since     1.0.0
     * @access    private
	 */
    private function load_admin_class()
    {
        require_once plugin_dir_path(dirname( __FILE__ )) . 'admin/admin.php';
        $admin_core = new Yourstyle_Plus_Companion_Plugin_Admin($this->plugin_name, $this->version);
    }

	/**
	 * Load the Admin style of the plugin.
	 *
	 * @since     1.0.0
     * @access    private
	 */
    public function load_admin_style()
    {
        wp_register_style( 'ysp-admin-style', plugins_url( '../admin/ysp-style.css' , __FILE__ ) );
    }

	/**
	 * Load the Yourstyle+ Add-on Classes
	 *
	 * @since     1.0.0
     * @access    private
	 */
    private function load_addons()
    {

        /** 
         *  Admin Column IDs
         * 
         *  Add the ID column to Posts, Pages and Media
         * 
         *  @since     1.1.0
         */
        if (get_option('ysp-addon-admin-column-ids') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/admin-column-ids.php';
        }

        /** 
         *  Duplicate Menu
         * 
         *  Adds a Duplicate Menu button to the Menu page
         * 
         *  @since     1.0.0
         */
        if (get_option('ysp-addon-duplicate-menu') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/duplicate-menu.php';
        }

        /** 
         *  Duplicate Posts
         * 
         *  Adds a Duplicate Post button on Post, Pages and Custom Post Type Archives in the Admin Area.
         * 
         *  @since     1.0.0
         */
        if (get_option('ysp-addon-duplicate-post') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/duplicate-post.php';
        }
        
        /** 
         *  Hang Nav
         * 
         *  Adds an optional Hang Nav to the website
         * 
         *  @since     1.0.0
         */
        if (get_option('ysp-addon-hang-nav') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/hang-nav.php';
        }

        /** 
         *  SVG Upload
         * 
         *  Lets you upload SVG files to the Media Library
         * 
         *  @since     1.0.0
         */
        if (get_option('ysp-addon-svg-upload') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/svg-upload.php';
        }

        /** 
         *  Top Header
         * 
         *  Adds the ability to create a top header with widgets
         * 
         *  @since     1.0.0
         */
        if (get_option('ysp-addon-top-header') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/top-header.php';
        }

        /** 
         *  Widget Accordion
         * 
         *  Adds a component with a widget with accordion functionalities.
         * 
         *  @since     1.6.0
         */
        if (get_option('ysp-addon-widget-accordion') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/widget-accordion.php';
        }

        /** 
         *  Widget CSS Classes
         * 
         *  Adds the option to add a CSS Class to widgets
         * 
         *  @since     1.0.0
         */
        if (get_option('ysp-addon-widget-css-classes') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/widget-css-classes.php';
        }

        /** 
         *  Widget Shortcode
         * 
         *  Output widgets using a simple Shortcode
         * 
         *  @since     1.5.0
         */
        if (get_option('ysp-addon-widget-shortcode') == "1") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/class-widget-shortcode.php';
            if (class_exists('YSP_Widget_Shortcode')) {
                YSP_Widget_Shortcode::get_instance();
            }
        }

        /** 
         *  Mediabox Shortcode
         * 
         *  Enables the mediabox shortcode and vc component.
         * 
         *  @since     1.7.5
         */
        if (get_option('ysp-addon-mediabox-shortcode') == "0") {
            require_once plugin_dir_path(dirname( __FILE__ )) . 'includes/addons/mediabox-shortcode.php';
        }        
    }

}